"""Launches and monitors transcoding program.

$LastChangedDate: 2009-11-26 13:57:48 -0800 (Thu, 26 Nov 2009) $
$Rev: 27 $

"""


import datetime
import math
import optparse
import os
import pythoncom
import StringIO
import subprocess
import sys
import threading
import time
import urllib2

from program_options import *


if os.name.lower() == 'nt':
    from win32com.client import GetObject
    import win32process


temp_output_extension = 'transcoding'

output_extension = None

default_service_wrapper_process_name = 'wrapper'
if os.name.lower() == 'nt':
    default_service_wrapper_process_name += '.exe'

default_sage_process_name = 'SageTV'
if os.name.lower() == 'nt':
    default_sage_process_name += '.exe'

mplayer_executable_name = 'mplayer'
if os.name.lower() == 'nt':
    mplayer_executable_name += '.exe'
mplayer_executable_path = os.path.join(os.path.dirname(sys.argv[0]),
                                       mplayer_executable_name)


def get_process_pid(process_name):
    if os.name.lower() == 'nt':
        wmi = GetObject('winmgmts:')
        processes = wmi.InstancesOf('Win32_Process')
        process_name = process_name.lower()

        for process in processes:
            if process.Properties_('Name').Value.lower() == process_name:
                return process.Properties_('ProcessId').Value
    else:
        raise Exception('This script currently only supports Windows')

    return None


def is_alive(pid):
    if pid is None:
        return False

    if os.name.lower() == 'nt':
        wmi = GetObject('winmgmts:')
        processes = wmi.InstancesOf('Win32_Process')

        for process in processes:
            if process.Properties_('ProcessId').Value == pid:
                return True
    else:
        raise Exception('This script currently only supports Windows')

    return False


def print_to_stdout_generator(process):
    process_pid = process.pid
    def local_is_alive():
        return process.poll() is None and is_alive(process_pid)

    def print_to_stdout_wrapper(process):
        if os.name.lower() == 'nt':
            # Initialize COM in this thread
            pythoncom.CoInitialize()

        print_to_stdout(process, local_is_alive)

    return print_to_stdout_wrapper


class DoubleLog(object):

    def __init__(self, file1, file2, flush_file1=False, flush_file2=False):
        self.file1 = file1
        self.file2 = file2
        self.flush_file1 = flush_file1
        self.flush_file2 = flush_file2

    def close(self):
        self.file1.close()
        self.file2.close()

    def closed(self):
        return self.file1.closed()

    def encoding(self):
        return self.file1.encoding()

    def fileno(self):
        return self.file1.fileno()
        
    def flush(self):
        self.file1.flush()
        self.file2.flush()

    def isatty(self):
        return self.file1.isatty()

    def mode(self):
        return self.file1.mode()

    def name(self):
        return self.file1.name()

    def newlines(self):
        return self.file1.newlines()

    def next(self):
        return self.file1.next()

    def seek(self, *args):
        self.file1.seek(*args)

    def softspace(self):
        return self.file1.softspace()
        
    def tell(self):
        return self.file1.tell()

    def truncate(self, *args):
        return self.file1.truncate(*args)

    def write(self, s):
        self.file1.write(s)
        self.file2.write(s)

        if self.flush_file1:
            self.file1.flush()
        if self.flush_file2:
            self.file2.flush()

    def writelines(self, arg1):
        self.file1.write(arg1)
        self.file2.write(arg1)


def get_python_log_filename(input_filename):
    return os.path.splitext(input_filename)[0] + os.extsep + \
        get_python_log_extension()


def main():
    start_time = datetime.datetime.now()

    parser = optparse.OptionParser()
    parser.add_option('--max-duration-percentage',
                      dest='max_duration_percentage',
                      type='float', default=-1,
                      help='Max elapsed time (percentage of input file)' \
                           '  to allow ComSkip to run')

    parser.add_option('--perform-service-wrapper-process-check',
                      dest='perform_service_wrapper_process_check',
                      action="store_true", default=False,
                      help='Whether or not to check for a locally running' \
                           ' service wrapper process.  The name of the' \
                           ' process is specified in' \
                           ' --service-wrapper-process-name')
    parser.add_option('--service-wrapper-process-name',
                      dest='service_wrapper_process_name',
                      help='Name of service wrapper process (defaults to' \
                           ' wrapper.exe on Windows, wrapper on linux).' \
                           '   Only used if' \
                           ' --perform-service-wrapper-process-check is' \
                           ' defined')

    parser.add_option('--perform-local-sage-process-check',
                      dest='perform_local_sage_process_check',
                      action="store_true", default=False,
                      help='Whether or not to check for a locally running' \
                           ' SageTV process.  The name of the process is' \
                           ' specified in --sage-process-name')
    parser.add_option('--sage-process-name', dest='sage_process_name',
                      help='Name of sage process (defaults to SageTV.exe on' \
                           ' Windows, SageTV on linux).  Only used if' \
                           ' --perform-local-sage-process-check is defined')

    parser.add_option('--perform-url-sage-process-check',
                      dest='perform_url_sage_process_check',
                      action="store_true", default=False,
                      help='Whether or not to check for a running SageTV' \
                           ' instance using a URL.  The url is specified in' \
                           ' --sage-process-check-url')
    parser.add_option('--sage-process-check-url',
                      dest='sage_process_check_url',
                      help='URL to read to check if SageTV is running.  If' \
                           ' the URL cannot be read it is assumed SageTV' \
                           " has terminated.  If the URL's contents have" \
                           ' changed it is assumed SageTV has restarted.' \
                           '  Either way this process will terminate.' \
                           '  Only used if --perform-url-sage-process-check' \
                           ' is defined')
    parser.add_option('--sage-process-check-url-timeout',
                      dest='sage_process_check_url_timeout',
                      type='float', default=20,
                      help='Timeout for URL check (defaults to 20)')

    parser.add_option('--fail-if-output-not-smaller',
                      dest='fail_if_output_not_smaller',
                      type='int', default=0,
                      help='Whether or not script will report failure if' \
                           ' output file is not smaller than input file' \
                           ' (defaults to 0)')

    parser.add_option('--min-duration',
                      dest='min_duration',
                      type='float', default=get_default_min_duration(),
                      help='Mininum duration of video (in seconds) to' \
                           ' process or -1 to process all videos (defaults' \
                           ' to %d)' % get_default_min_duration())
    parser.add_option('--input-file', dest='input_filename',
                      help='Input filename')
    options, args = parser.parse_args()

    input_filename = options.input_filename

    if not input_filename:
        parser.error('input_filename is required')
        sys.exit(1)

    python_log_filename = get_python_log_filename(input_filename)

# Temp (can speed up testing)
    if os.path.exists(python_log_filename):
        os.remove(python_log_filename)
# end temp

    # If a python_log file exists see if it growing, if it is, another process
    # is operating on this input file, exit
    if os.path.exists(python_log_filename):
        print 'Log file exists, sleeping 20 seconds to see if it grows' \
              ' (to see if another process is working on the input file)'

        file_size1 = os.path.getsize(python_log_filename)
        time.sleep(20)
        file_size2 = os.path.getsize(python_log_filename)
        
        if file_size1 != file_size2:
            print 'Exiting, another process appears to be working on "%s"' % \
                  input_filename
            return False

        print 'Continuing as no other process appears to be working on' \
              ' "%s" (log file size before: %d, after %d)' % \
              (input_filename, file_size1, file_size2)

        print 'Removing "%s"' % python_log_filename
        os.remove(python_log_filename)

    rval = False
    original_stdout = sys.stdout
    log_file = open(python_log_filename, 'w')
    try:
        # Change stdout to write to stdout and to log file
        sys.stdout = DoubleLog(sys.stdout, log_file, True, True)

        print 'Start time: %s' % start_time

        print 'Beginning process: %s' % ' '.join(sys.argv)
        try:
            rval = main_helper(options)
        except Exception, e:
            import traceback
            print 'Error calling main_helper: error:%s\ntraceback:%s' % \
                  (e, traceback.format_exc())
    finally:
        time_string = to_time_string(datetime.datetime.now() - start_time)
        print '%s ran for %s' % (get_process_name(), time_string)

        if rval:
            print 'Success'
        else:
            print 'Failure'

        sys.stdout.flush()
        sys.stdout = original_stdout
        log_file.close()

        # If script succeeded, remove log file
        if rval:
            print 'Removing "%s"' % python_log_filename
            try:
                os.remove(python_log_filename)
            except Exception, e:
                print 'Unable to delete "%s", error:%s' % \
                      (python_log_filename, e)

    if not rval:
        return 1
    return 0


def check_url(args_dict):
    print 'Check url thread: starting thread'

    try:
        timeout = int(args_dict['timeout'])
    except:
        timeout = 10

    previous_value = None
    try:
        while not args_dict['thread_should_terminate']:
            start_time = datetime.datetime.now()

            # Get value
            try:
                value = urllib2.urlopen(args_dict['url'],
                                        timeout=timeout).read()
            except urllib2.URLError, e:
                # Ignore time outs, this script is only worried about
                # duplicate apps getting launched and that will
                # only happen when Sage restarts
                s = 'URL error @ %s: %s from URL: %s' % \
                    (start_time, e, args_dict['url'])

                if 'urlopen error timed out' in ('%s' % e):
                    print s
                    continue

                args_dict['failure_message'] = s
                args_dict['value_has_changed'] = True
                break

            # Set value
            args_dict['value'] = value

            # If value has changed, stop
            if previous_value is not None and \
               (previous_value != args_dict['value']):
                args_dict['failure_message'] = \
                    'value changed, previous_value: %s, value: %s' \
                    ', from URL: %s' % \
                    (previous_value, args_dict['value'], args_dict['url'])
                args_dict['value_has_changed'] = True
                break

            previous_value = args_dict['value']

            # Wait remainder of timeout
            time_delta = (datetime.datetime.now() - start_time).seconds
            if timeout > time_delta:
                sleep_time = int(math.ceil(timeout - time_delta))
# For debugging only
#                print 'Check url thread: sleeping for %d seconds before' \
#                      ' loading URL again' % sleep_time
# End for debugging only
                for i in range(sleep_time):
                    if args_dict['thread_should_terminate']:
                        break
                    time.sleep(1)
    except Exception, e:
        print 'Check url thread: exception: %s' % e

        args_dict['failure_message'] = 'Exception: %s' % e
        args_dict['value_has_changed'] = True

    print 'Check url thread: thread finished'


def main_helper(options):
    rval = False
    try:
        rval = main_helper_helper(options)
        return rval
    finally:
        print 'Cleaning up before exiting'
        clean_up(options.input_filename, not rval)
        print 'Done cleaning up'


def main_helper_helper(options):
    input_filename = options.input_filename

    if not input_filename:
        raise Exception('Input filename not provided in "%s"' % sys.argv[1:])

    if not os.path.exists(input_filename):
        raise Exception('Input filename "%s" does not exist' % input_filename)

    # Only operate on certain types:
    input_extension = os.path.splitext(input_filename)[1]
    if input_extension.startswith(os.extsep):
        input_extension = input_extension[len(os.extsep):]
    if input_extension not in get_accepted_input_extensions():
        print 'Input file\'s ("%s") extension "%s" is not in %r' % \
              (input_filename, input_extension,
               get_accepted_input_extensions())
        return False

    if not os.path.isabs(input_filename):
        input_filename = os.path.abspath(input_filename)

    if options.perform_service_wrapper_process_check:
        if options.service_wrapper_process_name:
            service_wrapper_process_name = \
                options.service_wrapper_process_name
        else:
            service_wrapper_process_name = \
                default_service_wrapper_process_name
        if os.name.lower() == 'nt':
            service_wrapper_process_name = \
                service_wrapper_process_name.lower()

        service_wrapper_process_pid = \
            get_process_pid(service_wrapper_process_name)
        if service_wrapper_process_pid is None:
            raise Exception('Unable to find process "%s"' % \
                            service_wrapper_process_name)

        print '%s has the pid %s' % (service_wrapper_process_name,
                                     service_wrapper_process_pid)
    else:
        service_wrapper_process_name = None

    if options.perform_local_sage_process_check:
        if options.sage_process_name:
            sage_process_name = options.sage_process_name
        else:
            sage_process_name = default_sage_process_name
        if os.name.lower() == 'nt':
            sage_process_name = sage_process_name.lower()

        sage_process_pid = get_process_pid(sage_process_name)
        if sage_process_pid is None:
            raise Exception('Unable to find process "%s"' % sage_process_name)

        print '%s has the pid %s' % (sage_process_name, sage_process_pid)
    else:
        sage_process_name = None

    media_stats_dict = get_media_stats_dict(input_filename)
    
    if not media_stats_dict.get('ID_LENGTH'):
        print 'Unable to get duration for input file "%s"' \
              ', media_stats_dict=%r' % (input_filename, media_stats_dict)
        return False

    try:
        duration = float(media_stats_dict['ID_LENGTH'])
    except:
        duration = 0

    if options.min_duration > 0 and duration < options.min_duration:
        print 'Input file duration of %s is shorter than minimum duration' \
              ' of %s' % (to_time_string(duration), options.min_duration)
        return True

    print 'Input file has a duration of %s' % to_time_string(duration)

    # Print out stats
    print '\nBegin input file stats:'
    for key, value in media_stats_dict.items():
        print '%s=%s' % (key, value)
    print 'End input file stats\n'

    executable_path = get_executable_path()

    # Calculate max_elapsed_time based on duration and max_duration_percentage
    max_elapsed_time = duration * options.max_duration_percentage / 100

    print 'Cleaning up before starting'
    clean_up(input_filename, True)
    print 'Done cleaning up'

    temp_output_filename = os.path.splitext(input_filename)[0] + os.extsep + \
        temp_output_extension

    start_time = datetime.datetime.now()
    requires_kill = False
    requires_kill_rval = False
    kill_message = None

    process_pid = None
    process = None
    check_url_thread_args = {}
    try:
        # If performing URL check, launch URL check thread and wait for value
        # to be set
        if options.perform_url_sage_process_check:
            check_url_thread_args = \
                {'url': options.sage_process_check_url, \
                 'timeout': options.sage_process_check_url_timeout,
                 'thread_should_terminate': False,
                 'value': None,
                 'value_has_changed': False,
                 'failure_message': ''
                }
            check_url_thread = threading.Thread(target=check_url,
                                                args=(check_url_thread_args,))
            check_url_thread.start()

            # Wait up to 1 minute for first value
            for i in range(12):
                if not check_url_thread.is_alive() or \
                   check_url_thread_args['value'] is not None:
                    break
                time.sleep(5)

            if check_url_thread_args['value_has_changed']:
                print 'URL check failed with message "%s"' % \
                      check_url_thread_args['failure_message']
                return False

        global output_extension
        output_extension, base_executable_options = \
            get_output_extension_options(media_stats_dict)
        print 'get_options() returned %r' % \
              base_executable_options

        executable_options = add_input_and_output_filenames(\
            base_executable_options, input_filename, temp_output_filename)

        print 'Launching %s with options "%s"' % \
              (get_process_name(), executable_options)
        if max_elapsed_time > 0:
            print '(will run for a maximum of %s)' % \
                  to_time_string(max_elapsed_time)
        sys.stdout.flush()

        if os.name.lower() == 'nt':
            creationflags = win32process.BELOW_NORMAL_PRIORITY_CLASS
        else:
            creationflags = 0

        process = subprocess.Popen([executable_path] + executable_options, \
            bufsize=1, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
            creationflags=creationflags)
        process_pid = process.pid
        print '%s pid: %s' % (get_process_name(), process_pid)

        stdout_reader_thread = threading.Thread(\
            target=print_to_stdout_generator(process), args=(process,))
        stdout_reader_thread.start()

        # Loop until process ends or Sage is shut down
        while process.poll() is None and is_alive(process_pid):
            if max_elapsed_time > 0:
                now = datetime.datetime.now()
                if (now - start_time) > \
                   datetime.timedelta(seconds=max_elapsed_time):
                    kill_message = 'Process will be killed because it has' \
                        ' run longer than %s' % \
                        to_time_string(max_elapsed_time)
                    requires_kill = True
                    break

            if options.perform_service_wrapper_process_check:
                pid = get_process_pid(service_wrapper_process_name)
                if pid is None:
                    kill_message = 'Process will be killed because "%s" is' \
                                   ' no longer running' % \
                                   service_wrapper_process_name
                    requires_kill = True
                    break

                if pid != service_wrapper_process_pid:
                    kill_message = 'Process will be killed because "%s" ' \
                                   ' has been restarted' % \
                                   service_wrapper_process_name
                    requires_kill = True
                    break

            if options.perform_local_sage_process_check:
                pid = get_process_pid(sage_process_name)
                if pid is None:
                    kill_message = 'Process will be killed because "%s" is' \
                                   ' no longer running' % sage_process_name
                    requires_kill = True
                    break

                if pid != sage_process_pid:
                    kill_message = 'Process will be killed because "%s" ' \
                                   ' has been restarted' % sage_process_name
                    requires_kill = True
                    break

            try:
                # If performing URL check, see if value has changed
                if options.perform_url_sage_process_check:
                    # If a value has been found
                    if check_url_thread_args['value_has_changed']:
                        kill_message = 'Process will be killed because' \
                            ' "%s"' % \
                            check_url_thread_args['failure_message']
                        requires_kill = True
                        break
            except Exception, e:
                print 'Check url exception: %s' % e
                requires_kill = True
                break

            if not os.path.exists(input_filename):
                print 'Input filename "%s" no longer exist' % input_filename
                requires_kill = True
                requires_kill_rval = True # Kill but return True
                break

            time.sleep(5)
            time.sleep(5)

        print '\n%s loop completed\n' % get_process_name()

        if requires_kill and is_alive(process_pid):
            print '\n\n'
            print kill_message
            process.kill()

            # Wait up to 1 minute for process to die
            print 'Waiting for process to die'
            for i in range(12):
                if not is_alive(process_pid):
                    break
                time.sleep(5)

            if not is_alive(process_pid):
                print '\n\n'
                print 'Process is dead'
            else:
                print 'Process could not be killed, pid:%s' % process_pid

            # Wait up to 1 minute for stdout thread to finish
            print 'Waiting for stdout worker thread to complete'
            for i in range(12):
                if not stdout_reader_thread.is_alive():
                    break
                time.sleep(5)

            if not stdout_reader_thread.is_alive():
                print 'Worker thread has completed'
            else:
                print 'Worker thread did not complete (log may be incomplete)'

            print '\nRepeating kill message: %s' % kill_message
            return requires_kill_rval

        time_string = to_time_string(datetime.datetime.now() - start_time)
        print '%s ran for %s' % (get_process_name(), time_string)

        print 'Process "%s" completed with returncode %s' % \
              (get_executable_name(), process.returncode)

        if not check_process_return_code(process.returncode):
            print 'Process failed with return code %s' % process.returncode
            return False

        print 'Done calling "%s"' % get_executable_name()
    except Exception, e:
        print 'Exception calling "%s", error: %s' % (get_executable_name(), e)

        # Signal check url thread to terminate and wait (if necessary)
        if options.perform_url_sage_process_check:
            check_url_thread_args['thread_should_terminate'] = True
            # Wait up to 1 minute for url check thread to finish
            print 'Waiting for check url thread to complete'
            for i in range(12):
                if not check_url_thread.is_alive():
                    break
                time.sleep(5)

        if is_alive(process_pid):
            process.kill()

        raise

    # Signal check url thread to terminate and wait (if necessary)
    if options.perform_url_sage_process_check:
        check_url_thread_args['thread_should_terminate'] = True
        # Wait up to 1 minute for url check thread to finish
        print 'Waiting for check url thread to complete'
        for i in range(12):
            if not check_url_thread.is_alive():
                break
            time.sleep(5)

    if not os.path.exists(temp_output_filename):
        print 'Failure: no output file was found'
        return False

    print 'Verifying duration of output file "%s"' % temp_output_filename
    output_media_stats_dict = get_media_stats_dict(temp_output_filename)
    
    # Print out stats
    print '\nBegin output file stats:'
    for key, value in output_media_stats_dict.items():
        print '%s=%s' % (key, value)
    print 'End output file stats\n'

    if not output_media_stats_dict.get('ID_LENGTH'):
        print 'Unable to get duration for output file "%s"' % \
              temp_output_filename
        return False

    try:
        output_duration = float(output_media_stats_dict['ID_LENGTH'])
    except:
        output_duration = 0

    if (abs(duration - output_duration) / duration * 100) > \
       get_max_output_duration_delta_percentage():
        print "Output file's (\"%s\") duration differs by more than" \
              ' %s%%' % (temp_output_filename,
                         get_max_output_duration_delta_percentage())
        print 'Input file duration:%s' % to_time_string(duration)
        print 'Output file duration:%s' % to_time_string(output_duration)
        return False

    print 'Output file has a duration of %s' % to_time_string(output_duration)

    bytes_per_mb = 1048576
    
    input_filesize = os.path.getsize(input_filename)
    input_filesize_mb = float(input_filesize) / bytes_per_mb
    print 'Input file size: %d bytes (%.2fMB)' % \
          (input_filesize, input_filesize_mb)

    output_filesize = os.path.getsize(temp_output_filename)
    output_filesize_mb = float(output_filesize) / bytes_per_mb
    print 'Output file size: %d bytes (%.2fMB)' % \
          (output_filesize, output_filesize_mb)

    if output_filesize != input_filesize:
        size_diff = abs(input_filesize - output_filesize)
        size_diff_mb = float(size_diff) / bytes_per_mb

        if output_filesize < input_filesize:
            change_description_string = 'savings'
        else:
            change_description_string = 'increase'
        print '(size %s: %d bytes or %.2fMB)' % (change_description_string,
                                                 size_diff, size_diff_mb)

    if options.fail_if_output_not_smaller and \
       output_filesize >= input_filesize:
        print 'Exiting because output file is not smaller than input and' \
              ' --fail-if-output-not-smaller is on'
        return False

    # Rename temp output file to actual output file
    #    "XXX.<temp_output_extension>" -> "XXX.<output_extension>"
    output_filename = os.path.splitext(input_filename)[0] + os.extsep + \
        output_extension
    print 'Renaming "%s" to "%s"' % (temp_output_filename, output_filename)
    os.rename(temp_output_filename, output_filename)
    return True


def get_media_stats_dict(input_filename):
    options = ['-vo', 'null', '-nosound', '-identify',
               '-frames', '0', input_filename]

    if os.name.lower() == 'nt':
        creationflags = win32process.BELOW_NORMAL_PRIORITY_CLASS
    else:
        creationflags = 0

    process = subprocess.Popen([mplayer_executable_path] + options, \
        stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        creationflags=creationflags)
    
    lines = []
    process_pid = process.pid
    while process.poll() is None and is_alive(process_pid):
        line = process.stdout.readline()
        if not line:
            time.sleep(1)
            continue
        lines.append(line)

    while True:
        line = process.stdout.readline()
        if not line:
            break
        lines.append(line)

    value_prefix = 'ID_'

    media_stats_dict = {}
    #for line in process_stdout.split('\n'):
    for line in lines:
        line = line.strip()
        if line.startswith(value_prefix):
            key, value = line.split('=', 1)
            key = key.strip()
            value = value.strip()
            try:
                if '.' in value:
                    value = float(value)
                else:
                    value = int(value)
            except:
                pass
            media_stats_dict[key] = value

    if not media_stats_dict:
        process_stderr = process.stderr.read()
        if process_stderr:
            raise Exception('Unable to get media stats for file "%s"' \
                            ', error:%s' % (input_filename, process_stderr))
        else:
            raise Exception('Unable to get media stats for file "%s"')

    return media_stats_dict


def to_time_string(time_delta):
    if isinstance(time_delta, datetime.timedelta):
        seconds = time_delta.seconds + (time_delta.microseconds / 1000000.0)
    else:
        seconds = time_delta

    hours, seconds = divmod(seconds, 3600)
    minutes, seconds = divmod(seconds, 60)

    parts = []

    if hours:
        s = '%d hour' % hours
        if hours != 1:
            s += 's'
        parts.append(s)

    if minutes:
        s = '%d minute' % minutes
        if minutes != 1:
            s += 's'
        parts.append(s)
        
    if seconds:
        s = '%.2f second' % seconds
        if seconds != 1:
            s += 's'
        parts.append(s)

    return ', '.join(parts) or '0 seconds'


def clean_up(input_filename, make_very_clean):
    # Delete "XXX.<temp_output_extension>"
    filename = os.path.splitext(input_filename)[0] + os.extsep + \
        temp_output_extension
    if os.path.exists(filename):
        print 'Removing "%s"' % filename
        try:
            os.remove(filename)
        except Exception, e:
            print 'Unable to delete "%s", error:%s' % (filename, e)

    # On very clean delete "XXX.<output_extension>"
    if output_extension is not None and make_very_clean:
        filename = os.path.splitext(input_filename)[0] + \
            os.extsep + output_extension
        if os.path.exists(filename):
            print 'Removing "%s"' % filename
            try:
                os.remove(filename)
            except Exception, e:
                print 'Unable to delete "%s", error:%s' % (filename, e)


if __name__ == '__main__':
    sys.exit(main())
